clear;
dg = 180 / pi;

info = read_envihdr('Land_Cover_CMG_2018.hdr');

lines = info.lines;
samples = info.samples;
format = info.format;

fid = fopen('Land_Cover_CMG_2018.dat','r');
Landuse = fread(fid,[lines samples],[format '=>' format]);
fclose(fid);

water = (Landuse == 0);

% Assumed leaf optical data in R and N band

rhoR = 0.02;
tauR = 0.00;
rhoN = 0.52;
tauN = 0.44;

S = 1.2;          % default soil spectral slope (NIR/red)

fid1 = fopen('BoutMin.dat','r');
fid2 = fopen('BoutMax.dat','r');

Npix = 3600 * 7200;

% Read 16 bit integer data in 7 MODIS bands

Bmin = fread(fid1,[Npix 7],'int16=>int16');
Bmax = fread(fid2,[Npix 7],'int16=>int16');

% Red and NIR are in bands 1 and 2. Convert to single for calculation of NDVI 

Rmin = single(Bmin(:,1)); Nmin = single(Bmin(:,2));
Rmax = single(Bmax(:,1)); Nmax = single(Bmax(:,2));

NDVImin = reshape((Nmin - Rmin) ./ (Nmin + Rmin),3600,7200);
NDVImin(water) = 0;
NDVImax = reshape((Nmax - Rmax) ./ (Nmax + Rmax),3600,7200);
NDVImax(water) = 0;

[hcb,ha1,ha2] = dualplot(NDVImin,NDVImax,[0 1],'NDVI');

ha1.Title.String = '(a) NDVI at time of NDVImin';
ha2.Title.String = '(b) NDVI at time of NDVImax';
saveas(gcf,'NDVI_minmax_2.emf','meta');

% Negative or zero values are considered not valid

val = Rmin>0 & Rmax>0 & Nmin>0 & Nmax>0;

% Generate direct LUTs for LAI, soil brightness and fAPAR

% 2D grids of red and NIR reflectance at 0.001 sampling density 

R = ones(1000,1) * (0:899)/1000 + .0005;
N = (0:999)'/1000 * ones(1,900) + .0005;

nr = size(R,1);
nc = size(R,2);
ne = nr * nc;

% Reshape to long vectors

R = reshape(R,ne,1); 
N = reshape(N,ne,1); 

tic;   % Measure time needed for LUT generation

% Model inversion with Models I - III

[LAI,SB,fAP] = RedNIR_inv_M3(rhoR,tauR,rhoN,tauN,S,R,N);

disp(num2str(toc));

% Convert single red and NIR reflectances into 32 bit integers and form
% indices for images of min and max NDVI

indexmin = int32(Rmin(val)) * 1000 + int32(Nmin(val));
indexmax = int32(Rmax(val)) * 1000 + int32(Nmax(val));

L_im_min = zeros(Npix,1);
B_im_min = zeros(Npix,1);
F_im_min = zeros(Npix,1);
L_im_max = zeros(Npix,1);
B_im_max = zeros(Npix,1);
F_im_max = zeros(Npix,1);

tic;   % measure time to apply direct LUT to one scene

% This is the application of the DLUT to the min NDVI image as a vector

L_im_min(val) = LAI(indexmin);
B_im_min(val) = SB(indexmin);
F_im_min(val) = fAP(indexmin);

disp(num2str(toc));

% This is the application of the DLUT to the max NDVI image as a vector

L_im_max(val) = LAI(indexmax);
B_im_max(val) = SB(indexmax);
F_im_max(val) = fAP(indexmax);

% Reshaping to CMG images

im1 = reshape(F_im_min,3600,7200);
im2 = reshape(F_im_max,3600,7200);
[~,ha1,ha2] = dualplot(im1,im2,[0 1],'fAPAR');

ha1.Title.String = '(a) fAPAR at time of NDVImin';
ha2.Title.String = '(b) fAPAR at time of NDVImax';
saveas(gcf,'fAPAR_minmax_2.emf','meta');

im1 = reshape(L_im_min,3600,7200);
im2 = reshape(L_im_max,3600,7200);
[~,ha1,ha2] = dualplot(im1,im2,[0 4],'LAIeff');

ha1.Title.String = '(a) LAIeff at time of NDVImin';
ha2.Title.String = '(b) LAIeff at time of NDVImax';
saveas(gcf,'LAIeff_minmax_2.emf','meta');

im1 = reshape(B_im_min,3600,7200);
im2 = reshape(B_im_max,3600,7200);
[~,ha1,ha2] = dualplot(im1,im2,[0 0.6],'Soil brightness');

ha1.Title.String = '(a) SB at time of NDVImin';
ha2.Title.String = '(b) SB at time of NDVImax';
saveas(gcf,'SB_minmax_2.emf','meta');

