function [Leff,SB,fAPAR] = RedNIR_inv_M3_Hy(rhoR,tauR,rhoN,tauN,S,R,N)
        
    % R and N can be column vectors of length np (number of pixels)
       
    np = length(R);
    
    % Model I: Homogeneous turbid medium with varying LAI
      
    L = 0 : 0.1 : 8;
    
    vL = ones(1,length(L));
    vR = ones(length(R),1);
    
    [Rm,Am,rdds,tdds] = Hy_BHR(rhoR,tauR,
   
    bR = -(1 + R * Rrdd - (1 + vR * Rsqdd));
    bN = -(1 + N * Nrdd - (1 + vR * Nsqdd));
    
    cR = R * vL - vR * Rrdd;
    cN = N * vL - vR * Nrdd;
    
    Rs = -cR ./ bR * 2; 
    Ns = -cN ./ bN * 2; 
    
    X = Ns - S * Rs;
    
    [LAI1,Cv2,fC3,SB1,SB2,fAPAR1,fAPAR2] = deal(NaN(np,1));
    [val1,val2] = deal(false(np,1)); 
    
    for i = 1:np
        
        j = find(X(i,:) < 0,1);
        if ~isempty(j)
            if Rs(i,j) >= -1 && Rs(i,j) <= 1 && Ns(i,j) >= -1 && Ns(i,j) <= 1 && j>1
                Le = (-X(i,j)*L(j-1) + X(i,j-1)*L(j))/(X(i,j-1)-X(i,j));
                Rse = (-X(i,j)*Rs(i,j-1) + X(i,j-1)*Rs(i,j))/(X(i,j-1)-X(i,j));
                if Le >= 0 && Le <= 8 && Rse >= 0
                    LAI1(i) = Le;
                    SB1(i)  = Rse;
                    val1(i) = true;
                end
            else
                 if X(i,1) < 0
                    LAI1(i) = 0;
                    SB1(i)  = R(i);
                 end
            end
        end
        
        % Calculate fAPAR in the red
        
        ex  = exp(-mR * LAI1(i));
        rs  = SB1(i);
        D   = 1 - (ex * Rinf).^2; 
        rdd = (1 - ex.^2) .* Rinf ./ D;
        tdd = (1 - Rinf^2) * ex ./ D;
        
        fAPAR1(i) = (1 - rdd - tdd) * (1 + rs * tdd / (1 - rs * rdd));
        
    end
    
    % Model II: Homogeneous forests with varying crown density
    
    fC = 1; L = 8;
    
    Cv = 0 : 0.01 : 1;
    
    vCv = ones(1,length(Cv));
    vR = ones(length(R),1);

    eR = exp(-mR * L);    
    eN = exp(-mN * L);
    
    DR = 1 - (eR * Rinf).^2; 
    DN = 1 - (eN * Ninf).^2;
    
    Rrdd = (1 - eR.^2) .* Rinf ./ DR * Cv;
    Nrdd = (1 - eN.^2) .* Ninf ./ DN * Cv;
    
    Rtdd = 1 - (1 - (1 - Rinf^2) * eR ./ DR) * Cv;
    Ntdd = 1 - (1 - (1 - Ninf^2) * eN ./ DN) * Cv;
       
    Rsqdd = Rrdd.^2 - Rtdd.^2;
    Nsqdd = Nrdd.^2 - Ntdd.^2;
    
    aR = 0; 
    aN = 0;
    
    bR = -(1 + R * Rrdd - (1 + vR * Rsqdd));
    bN = -(1 + N * Nrdd - (1 + vR * Nsqdd));
    
    cR = R * vCv - vR * Rrdd;
    cN = N * vCv - vR * Nrdd;
    
    Rs = -cR ./ bR * 2.; 
    Ns = -cN ./ bN * 2.; 
    
    X = Ns - S * Rs;
    
    [rsR,rsN,R8,N8,fAPAR8] = deal(zeros(np,1));
       
    for i = 1:np
                
        j = find(X(i,:) < 0,1);
        if ~isempty(j)
            if Rs(i,j) >= -1 && Rs(i,j) <= 1 && Ns(i,j) >= -1 && Ns(i,j) <= 1 && j>1
                Cve = (-X(i,j)*Cv(j-1) + X(i,j-1)*Cv(j)) / (X(i,j-1) - X(i,j));
                Rse = (-X(i,j)*Rs(i,j-1) + X(i,j-1)*Rs(i,j)) / (X(i,j-1) - X(i,j));
                if Cve >= 0 && Cve <= 1 && Rse >= 0
                    Cv2(i) = Cve;
                    SB2(i) = Rse;
                    val2(i) = true;
                end
            else                
                if X(i,1) < 0
                    Cv2(i) = 0;
                    SB2(i)  = R(i);
                    val2(i) = false;
                end
            end
        end
        
        % Calculate fAPAR in the red for LAI = 8
        
        rsR(i)  = SB2(i);
        rsN(i)  = rsR(i) * S;
        
        DR   = 1 - (eR * Rinf).^2;
        DN   = 1 - (eN * Ninf).^2;
        
        Rrdd8 = (1 - eR.^2) .* Rinf ./ DR;
        Rtdd8 = (1 - Rinf^2) * eR ./ DR;
        
        rdd = Rrdd8 * Cv2(i);
        tdd = 1 - (1 - Rtdd8) * Cv2(i);
        
        fAPAR2(i) = (1 - rdd - tdd) * (1 + rsR(i) * tdd ./ (1 - rsR(i) * rdd));
        
        Nrdd8 = (1 - eN.^2) .* Ninf ./ DN;
        Ntdd8 = (1 - Ninf^2) * eN ./ DN;
        
        R8(i) = Rrdd8 + Rtdd8^2 * rsR(i) ./ (1 - Rrdd8 * rsR(i));
        N8(i) = Nrdd8 + Ntdd8^2 * rsN(i) ./ (1 - Nrdd8 * rsN(i));
        
        fAPAR8(i) = (1 - Rrdd8 - Rtdd8) * (1 + rsR(i) * Rtdd8 ./ (1 - rsR(i) * Rrdd8));
 
    end
    
    % Model III Heterogeneous pixel, dense turbid medium + bare soil
    
    fC = (N - S * R) ./ (N8 - S * R8);
    val3 = (fC >= 0 & fC <= 1);
    fC3(val3) = fC(val3);
    SB3 = (R - fC3 .* R8) ./ (1 - fC3);
    fAPAR3 = fAPAR8 .* fC3;
    
    sumval = val1 + val2 + val3;
    
    Leff = (LAI1 .* val1 + (Cv2 * 8) .* val2 + (fC3 * 8) .* val3) ./ sumval;
    SB = (SB1 .* val1 + SB2 .* val2 + SB3 .* val3) ./ sumval;
    fAPAR = (fAPAR1 .* val1 + fAPAR2 .* val2 + fAPAR3 .* val3) ./ sumval;
    
    NDVI = (N - R) ./ (N + R);
    
    % [LAI1 SB1 fAPAR1 Cv2 SB2 fAPAR2 fC3 SB3 fAPAR3]
       
end


